package immibis.ars.beams;

import java.util.Random;
import net.minecraft.server.AxisAlignedBB;
import net.minecraft.server.Block;
import net.minecraft.server.IBlockAccess;
import net.minecraft.server.Material;
import net.minecraft.server.ModLoader;
import net.minecraft.server.TileEntity;
import net.minecraft.server.World;

public class BlockBeam extends Block
{
    public static final int MAX_LENGTH = 8;

    public BlockBeam(int var1)
    {
        super(var1, 0, Material.REPLACEABLE_PLANT);
        this.setTextureFile("/immibis/ars/textures/tesla.png");
        this.a("immibis.ars.beams.BlockBeam");
        this.a(true);
        ModLoader.addName(this, "Wireless data beam");
        ModLoader.registerBlock(this);
    }

    /**
     * From the specified side and block metadata retrieves the blocks texture. Args: side, metadata
     */
    public int a(int var1, int var2)
    {
        return var2 < 6 ? 0 : 7;
    }

    /**
     * Is this block (a) opaque and (b) a full 1m cube?  This determines whether or not to render the shared face of two
     * adjacent blocks and also whether the player can attach torches, redstone wire, etc to this block.
     */
    public boolean a()
    {
        return false;
    }

    /**
     * Returns a bounding box from the pool of bounding boxes (this means this box can change after the pool has been
     * cleared to be reused)
     */
    public AxisAlignedBB e(World var1, int var2, int var3, int var4)
    {
        return null;
    }

    public static int directionFromMetadata(int var0)
    {
        return var0 % 6;
    }

    private void checkContinuity(World var1, int var2, int var3, int var4)
    {
        int var5 = var1.getData(var2, var3, var4);
        int var6 = directionFromMetadata(var5);
        byte var7 = 0;
        byte var8 = 0;
        byte var9 = 0;

        switch (var6)
        {
            case 0:
                var8 = -1;
                break;

            case 1:
                var8 = 1;
                break;

            case 2:
                var9 = -1;
                break;

            case 3:
                var9 = 1;
                break;

            case 4:
                var7 = -1;
                break;

            case 5:
                var7 = 1;
        }

        if (var7 == 0 && var8 == 0 && var9 == 0)
        {
            var1.setTypeId(var2, var3, var4, 0);
        }
        else
        {
            boolean var10 = false;
            int var11 = var2;
            int var12 = var3;
            int var13 = var4;
            int var14;

            for (var14 = 0; var14 < 8; ++var14)
            {
                var11 -= var7;
                var12 -= var8;
                var13 -= var9;

                if (var1.getTypeId(var11, var12, var13) != this.id || var1.getData(var11, var12, var13) != var5)
                {
                    TileEntity var15 = var1.getTileEntity(var11, var12, var13);
                    var10 = var15 instanceof TileBeamEmitter && ((TileBeamEmitter)var15).isOutputFace(var6);
                    break;
                }
            }

            if (!var10)
            {
                var1.setTypeId(var2, var3, var4, 0);
            }
            else if (var14 < 7)
            {
                int var18 = var2 + var7;
                int var16 = var3 + var8;
                int var17 = var4 + var9;

                if (var16 >= 0 && var16 < var1.getHeight() && var1.getTypeId(var18, var16, var17) == 0)
                {
                    var1.setTypeIdAndData(var18, var16, var17, this.id, var5);
                }
            }
        }
    }

    public boolean isBlockReplaceable(World var1, int var2, int var3, int var4)
    {
        return true;
    }

    /**
     * Lets the block know when one of its neighbor changes. Doesn't know which neighbor changed (coordinates passed are
     * their own) Args: x, y, z, neighbor blockID
     */
    public void doPhysics(World var1, int var2, int var3, int var4, int var5)
    {
        this.checkContinuity(var1, var2, var3, var4);
    }

    /**
     * Called whenever the block is added into the world. Args: world, x, y, z
     */
    public void onPlace(World var1, int var2, int var3, int var4)
    {
        this.checkContinuity(var1, var2, var3, var4);
    }

    /**
     * Returns the mobility information of the block, 0 = free, 1 = can't push but can move over, 2 = total immobility
     * and stop pistons
     */
    public int g()
    {
        return 2;
    }

    public int quantityDropped(int var1, int var2, Random var3)
    {
        return 0;
    }

    /**
     * Updates the blocks bounds based on its current state. Args: world, x, y, z
     */
    public void updateShape(IBlockAccess var1, int var2, int var3, int var4)
    {
        int var5 = directionFromMetadata(var1.getData(var2, var3, var4));
        float var10000 = 0.4375F;
        var10000 = 0.5625F;

        if (var5 != 0 && var5 != 1)
        {
            if (var5 != 2 && var5 != 3)
            {
                if (var5 != 4 && var5 != 5)
                {
                    this.a(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
                }
                else
                {
                    this.a(0.0F, 0.4375F, 0.4375F, 1.0F, 0.5625F, 0.5625F);
                }
            }
            else
            {
                this.a(0.4375F, 0.4375F, 0.0F, 0.5625F, 0.5625F, 1.0F);
            }
        }
        else
        {
            this.a(0.4375F, 0.0F, 0.4375F, 0.5625F, 1.0F, 0.5625F);
        }
    }
}
