package immibis.ars.beams;

import immibis.ars.beams.logictypes.ClassDetector;
import immibis.ars.beams.logictypes.Intersection;
import immibis.ars.beams.logictypes.Negate;
import immibis.ars.beams.logictypes.NothingDetector;
import immibis.ars.beams.logictypes.RedstoneInput;
import immibis.ars.beams.logictypes.Union;
import immibis.core.api.porting.SidedProxy;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import net.minecraft.server.Block;
import net.minecraft.server.EntityAnimal;
import net.minecraft.server.EntityArrow;
import net.minecraft.server.EntityBlaze;
import net.minecraft.server.EntityCaveSpider;
import net.minecraft.server.EntityChicken;
import net.minecraft.server.EntityCow;
import net.minecraft.server.EntityCreeper;
import net.minecraft.server.EntityEnderman;
import net.minecraft.server.EntityGhast;
import net.minecraft.server.EntityHuman;
import net.minecraft.server.EntityIronGolem;
import net.minecraft.server.EntityItem;
import net.minecraft.server.EntityLiving;
import net.minecraft.server.EntityMagmaCube;
import net.minecraft.server.EntityOcelot;
import net.minecraft.server.EntityPig;
import net.minecraft.server.EntityPigZombie;
import net.minecraft.server.EntityProjectile;
import net.minecraft.server.EntitySheep;
import net.minecraft.server.EntitySilverfish;
import net.minecraft.server.EntitySkeleton;
import net.minecraft.server.EntitySlime;
import net.minecraft.server.EntitySnowman;
import net.minecraft.server.EntitySquid;
import net.minecraft.server.EntityVillager;
import net.minecraft.server.EntityWolf;
import net.minecraft.server.EntityZombie;
import net.minecraft.server.IMonster;
import net.minecraft.server.Item;
import net.minecraft.server.ItemStack;

public abstract class LogicType
{
    private static Map types = new HashMap();
    private static Map items = new HashMap();
    public static final int ID_PLAYER = 0;
    public static final int ID_AND = 1;
    public static final int ID_OR = 2;
    public static final int ID_INVERT = 3;
    public static final int ID_BLANK = 4;
    public static final int ID_LIVING = 5;
    public static final int ID_REDSTONE = 6;
    public static final int ID_ANIMAL = 7;
    public static final int ID_MOB = 8;
    public static final int ID_ITEM = 9;

    public static LogicType get(int var0)
    {
        return (LogicType)types.get(Integer.valueOf(var0));
    }

    public static void register(int var0, String var1, LogicType var2, Item var3)
    {
        if (types.containsKey(Integer.valueOf(var0)))
        {
            throw new IllegalArgumentException("Logic card type " + var0 + " is already registered.");
        }
        else
        {
            if (var1 != null)
            {
                types.put(Integer.valueOf(var0), var2);
                SidedProxy.instance.addLocalization("immibis.ars.logiccard." + var0 + ".name", var1);
            }

            if (var3 != null)
            {
                items.put(var3, var2);
            }
        }
    }

    public static Set getAllTypeIDs()
    {
        return types.keySet();
    }

    public abstract EntityFilter createFilter(TileUpgradeUnit var1, EntityFilter[] var2);

    static void init()
    {
        register(0, "Player detector token", new ClassDetector(EntityHuman.class), (Item)null);
        register(1, "Intersection token", new Intersection(), (Item)null);
        register(2, "Union token", new Union(), (Item)null);
        register(3, "Difference token", new Negate(), (Item)null);
        register(4, "Blank token", new NothingDetector(), (Item)null);
        register(5, "Life detector token", new ClassDetector(EntityLiving.class), (Item)null);
        register(6, "Redstone input token", new RedstoneInput(), Item.REDSTONE);
        register(7, "Animal detector token", new ClassDetector(EntityAnimal.class), (Item)null);
        register(8, "Mob detector token", new ClassDetector(IMonster.class), (Item)null);
        register(9, "Item detector token", new ClassDetector(EntityItem.class), Item.byId[Block.COBBLESTONE.id]);
        register(-1, (String)null, new ClassDetector(EntityCreeper.class), Item.SULPHUR);
        register(-1, (String)null, new ClassDetector(EntityChicken.class), Item.EGG);
        register(-1, (String)null, new ClassDetector(EntityCow.class), Item.RAW_BEEF);
        register(-1, (String)null, new ClassDetector(EntityOcelot.class), Item.RAW_FISH);
        register(-1, (String)null, new ClassDetector(EntityPig.class), Item.PORK);
        register(-1, (String)null, new ClassDetector(EntitySheep.class), Item.byId[Block.WOOL.id]);
        register(-1, (String)null, new ClassDetector(EntitySquid.class), Item.WATER_BUCKET);
        register(-1, (String)null, new ClassDetector(EntityVillager.class), (Item)null);
        register(-1, (String)null, new ClassDetector(EntityEnderman.class), Item.ENDER_PEARL);
        register(-1, (String)null, new ClassDetector(EntityWolf.class), Item.BONE);
        register(-1, (String)null, new ClassDetector(EntityPigZombie.class), Item.GOLD_NUGGET);
        register(-1, (String)null, new ClassDetector(EntityBlaze.class), Item.BLAZE_ROD);
        register(-1, (String)null, new ClassDetector(EntityCaveSpider.class), (Item)null);
        register(-1, (String)null, new ClassDetector(EntityZombie.class), Item.ROTTEN_FLESH);
        register(-1, (String)null, new ClassDetector(EntityGhast.class), Item.GHAST_TEAR);
        register(-1, (String)null, new ClassDetector(EntityMagmaCube.class), Item.MAGMA_CREAM);
        register(-1, (String)null, new ClassDetector(EntitySilverfish.class), (Item)null);
        register(-1, (String)null, new ClassDetector(EntitySkeleton.class), Item.BOW);
        register(-1, (String)null, new ClassDetector(EntitySlime.class), Item.SLIME_BALL);
        register(-1, (String)null, new ClassDetector(EntitySnowman.class), Item.SNOW_BALL);
        register(-1, (String)null, new ClassDetector(EntityIronGolem.class), Item.IRON_INGOT);
        register(-1, (String)null, new ClassDetector(EntityArrow.class), Item.ARROW);
        register(-1, (String)null, new ClassDetector(EntityProjectile.class), (Item)null);
    }

    public static LogicType getForItem(ItemStack var0)
    {
        return var0 == null ? null : (var0.id == BeamsMain.itemLogicCard.id ? get(var0.getData()) : (LogicType)items.get(var0.getItem()));
    }
}
