package immibis.ars.beams;

import immibis.core.TileCombined;
import net.minecraft.server.EntityLiving;
import net.minecraft.server.NBTTagCompound;
import net.minecraft.server.NetworkManager;
import net.minecraft.server.Packet;
import net.minecraft.server.Packet132TileEntityData;
import net.minecraft.server.TileEntity;

public class TileBeamEmitter extends TileCombined
{
    public int outputFace;
    private TileBeamEmitter[] beamLinkCache = new TileBeamEmitter[6];

    /**
     * Overriden in a sign to provide the text
     */
    public Packet d()
    {
        Packet132TileEntityData var1 = new Packet132TileEntityData();
        var1.d = this.outputFace;
        var1.a = this.x;
        var1.b = this.y;
        var1.c = this.z;
        var1.lowPriority = true;
        return var1;
    }

    public void onDataPacket(NetworkManager var1, Packet132TileEntityData var2)
    {
        this.outputFace = var2.d;
    }

    public boolean isOutputFace(int var1)
    {
        return var1 == this.outputFace;
    }

    public Object getOutput()
    {
        return null;
    }

    public Object getOutput(int var1)
    {
        return var1 == this.outputFace ? this.getOutput() : null;
    }

    public int getBeamColour()
    {
        return 0;
    }

    public Object getInput(int var1)
    {
        if (this.beamLinkCache[var1] != null)
        {
            return this.beamLinkCache[var1].getOutput(var1 ^ 1);
        }
        else
        {
            byte var2 = 0;
            byte var3 = 0;
            byte var4 = 0;

            switch (var1)
            {
                case 0:
                    var3 = -1;
                    break;

                case 1:
                    var3 = 1;
                    break;

                case 2:
                    var4 = -1;
                    break;

                case 3:
                    var4 = 1;
                    break;

                case 4:
                    var2 = -1;
                    break;

                case 5:
                    var2 = 1;
            }

            int var5 = this.x + var2;
            int var6 = this.y + var3;
            int var7 = this.z + var4;
            int var8 = 0;

            while (this.world.getTypeId(var5, var6, var7) == BeamsMain.blockBeam.id && BlockBeam.directionFromMetadata(this.world.getData(var5, var6, var7)) == (var1 ^ 1))
            {
                var5 += var2;
                var6 += var3;
                var7 += var4;
                ++var8;

                if (var8 > 8)
                {
                    return null;
                }
            }

            TileEntity var9 = this.world.getTileEntity(var5, var6, var7);

            if (var9 instanceof TileBeamEmitter)
            {
                TileBeamEmitter var10 = (TileBeamEmitter)var9;
                this.beamLinkCache[var1] = var10;
                return var10.getOutput(var1 ^ 1);
            }
            else
            {
                return null;
            }
        }
    }

    protected void setupBeams()
    {
        this.setupBeam(this.outputFace);
    }

    public void onPlaced(EntityLiving var1, int var2)
    {
        this.outputFace = var2;
        this.setupBeams();
    }

    /**
     * Writes a tile entity to NBT.
     */
    public void b(NBTTagCompound var1)
    {
        super.b(var1);
        var1.setByte("outputFace", (byte)this.outputFace);
    }

    /**
     * Reads a tile entity from NBT.
     */
    public void a(NBTTagCompound var1)
    {
        super.a(var1);
        this.outputFace = var1.getByte("outputFace");
    }

    protected void setupBeam(int var1)
    {
        int var2 = var1 == 4 ? -1 : (var1 == 5 ? 1 : 0);
        int var3 = var1 == 0 ? -1 : (var1 == 1 ? 1 : 0);
        int var4 = var1 == 2 ? -1 : (var1 == 3 ? 1 : 0);
        int var5 = var1 + this.getBeamColour() * 6;
        int var6 = this.x;
        int var7 = this.y;
        int var8 = this.z;

        for (int var9 = 0; var9 < 8; ++var9)
        {
            var6 += var2;
            var7 += var3;
            var8 += var4;

            if (this.world.getTypeId(var6, var7, var8) != 0)
            {
                break;
            }

            this.world.setTypeIdAndData(var6, var7, var8, BeamsMain.blockBeam.id, var5);
        }
    }

    public void onBlockNeighbourChange()
    {
        this.setupBeams();

        for (int var1 = 0; var1 < 6; ++var1)
        {
            this.beamLinkCache[var1] = null;
        }
    }

    public int getTextureIndex()
    {
        return 0;
    }
}
